;; toggle-display-line-numbers in menu-bar.el should probably be set to a mode
(defgroup display-line-numbers nil
  "Show line numbers in the buffer."
  :group 'convenience)

(defcustom display-line-number-type t
  "Sets the type of line numbers to use in `display-line-number-mode'.
See `display-line-numbers' for value options."
  :group 'display-line-numbers
  :type 'symbol)

(defcustom display-line-number-inhibit-shrink nil
  "If non-nil, do not shrink line number width."
  :group 'display-line-numbers
  :type 'boolean)

(defcustom display-line-number-width-start nil
  "If non-nil, set initial line number width based on buffer contents.
`display-line-number-width' is set to the minimum width necessary
to display all line numbers in the buffer."
  :group 'display-line-numbers
  :type 'boolean)

(defun display-line-numbers-update-width ()
  (setq display-line-number-width
        (max (or display-line-number-width 1)
             (line-number-display-width))))

(define-minor-mode display-line-numbers-mode
  "Toggle display of line numbers in the buffer."
  :lighter nil
  (if display-line-numbers-mode
      (progn
        (when display-line-number-width-start
          (setq display-line-number-width
                (length (number-to-string
                         (count-lines (point-min) (point-max))))))
        (when display-line-number-inhibit-shrink
          (add-hook 'pre-command-hook #'display-line-numbers-update-width nil t))
        (setq display-line-numbers display-line-number-type))
    (remove-hook 'pre-command-hook #'display-line-numbers-update-width t)
    (setq display-line-numbers nil)))

(define-globalized-minor-mode global-display-line-numbers-mode
  display-line-numbers-mode
  (lambda ()
    (unless (or (minibufferp)
                ;; taken from linum.el
                (and (daemonp) (null (frame-parameter nil 'client))))
      (display-line-numbers-mode))))
